@@ -1,22 +1,17 @@
-# @(#)$Ident: Programs.pm 2014-01-01 22:41 pjf ;
-
 package Class::Usul::Config::Programs;
 
 use namespace::sweep;
-use version; our $VERSION = qv( sprintf '0.37.%d', q$Rev: 1 $ =~ /\d+/gmx );
 
 use Moo;
 use Class::Usul::Constants;
-use Class::Usul::Types      qw( ArrayRef Bool NonEmptySimpleStr
-                                NonZeroPositiveInt PositiveInt );
-use File::Basename          qw( basename );
-use File::DataClass::Types  qw( Path );
+use Class::Usul::Types     qw( ArrayRef Bool NonEmptySimpleStr
+                               NonZeroPositiveInt PositiveInt );
+use File::Basename         qw( basename );
+use File::DataClass::Types qw( Path );
 use File::HomeDir;
 
 extends q(Class::Usul::Config);
 
-has 'cache_ttys'   => is => 'ro',   isa => Bool, default => TRUE;
-
 has 'doc_title'    => is => 'ro',   isa => NonEmptySimpleStr,
    default         => 'User Contributed Documentation';
 
@@ -53,10 +48,6 @@ __END__
 
 Class::Usul::Config::Programs - Additional configuration attributes for CLI programs
 
-=head1 Version
-
-This documents version v0.37.$Rev: 1 $
-
 =head1 Synopsis
 
    package Class::Usul::Programs;
